import os
from charmhelpers.core import hookenv
from charms.reactive import (
    when,
    when_not,
    set_state,
    remove_state
)
from subprocess import (
    check_call,
    CalledProcessError,
    call,
    check_output,
    Popen
)

from charmhelpers.core.hookenv import (
    status_set,
    log,
    resource_get
)

import urllib.request as request
from time import sleep


def initialise_conf():
    try:
        f = open("/var/snap/apache-drill-spicule/current/drill/conf/drill-override.conf", "r")
        print("file exists")
        f.close()
        os.remove("/var/snap/apache-drill-spicule/current/drill/conf/drill-override.conf")
        f = open("/var/snap/apache-drill-spicule/current/drill/conf/drill-override.conf", "w")
    except FileNotFoundError:
        print("file does not exist...")
        f = open("/var/snap/apache-drill-spicule/current/drill/conf/drill-override.conf", "w")
        print("new file created.")
    return f


def start_template():
    cluster = hookenv.config()['cluster_id']
    temp = "drill.exec: {\n" \
        "\tcluster-id: \""+cluster+"\""
    return temp


def add_zookeeper(host, port):
    return host + ':' + port + ','


def zk_template(enabled, zookeepers):
    if not enabled:
        return ""

    portlist = ''
    for zk_unit in zookeepers.zookeepers():
        # for zk_unit  in zookeepers:
        portlist += add_zookeeper(zk_unit['host'], zk_unit['port'])
    portlist = portlist[:-1]

    return ",\n\tzk.connect: \"" + portlist + "\""


def auth_template(enabled, max_chain=3):
    if not enabled:
        return ""

    return ",\n\timpersonation: {\n\t\tenabled: true,\n\t\tmax_chain_user_hops: " + str(
        max_chain) + "\n\t}"


def auth_security(mechanisms, enabled):
    if not enabled:
        return ""

    return ",\n\tsecurity: {\n\t\t\tauth.mechanisms : [\"" + \
        mechanisms + "\"],\n\t\t}"


def security_user_auth_template(enabled,
                                profiles,
                                packages="org.apache.drill.exec.rpc.user.security",
                                impl="pam4j"
                                ):
    string = profiles.split(', ')
    temp = '['
    i = 0
    for profile in string:
        if (i < len(string) - 1):
            temp = temp + " \"" + profile + "\","
            i = i + 1
        else:
            temp = temp + " \"" + profile + "\" ]"

    if not enabled:
        return ""

    return ",\n\tsecurity.user.auth: {" \
        "\n\t\tenabled: true,\n\t\tpackages += \"" + packages + "\"," \
        "\n\t\timpl: \"" + impl + "\", \n\t\tpam_profiles: " + temp + "\n\t}"


def finish_template():
    temp = "\n}"
    return temp

def check_responsive(lc):
    for _ in range(lc):
        try:
            req = request.Request('http://localhost:8047')
            resp = request.urlopen(req)
            break
        except Exception:
            sleep(5)
            log("ping failed, retrying")

@when('drillbit.configured', 'zookeeper.ready')
def write_config_file(zookeeper):

    with open('/var/snap/apache-drill-spicule/current/drill/conf/drill-override.conf', 'r') as myfile:
        oldfile=myfile.read().replace('\n', '')
        myfile.close()

    conf_file = initialise_conf()
    text = []
    text.append(start_template())
    text.append(zk_template(True, zookeeper))
    text.append(auth_template(hookenv.config()['basic_auth']))
    text.append(auth_security(hookenv.config()['auth_mechanism'], hookenv.config()['basic_auth']))
    text.append(
        security_user_auth_template(
            hookenv.config()['basic_security_auth'],
            profiles=hookenv.config()['auth_profiles']))
    text.append(finish_template())

    for conf in text:
        conf_file.write(conf)
    conf_file.close()

    with open('/var/snap/apache-drill-spicule/current/drill/conf/drill-override.conf', 'r') as myotherfile:
        newfile=myotherfile.read().replace('\n', '')
        myotherfile.close()
        
    if newfile != oldfile:
        remove_state('drillbit.running')
        check_call('apache-drill-spicule.stop-drill-distributed', shell=True)
        check_call('apache-drill-spicule.start-drill-distributed', shell=True)
        check_responsive(10)
        status_set('active', 'Apache Drill up and running.')
        set_state('drillbit.running')