import uuid
import pytest
from juju.controller import Controller
import subprocess
import inspect
import requests
from time import sleep

test_run_nonce = uuid.uuid4().hex[-4:]



def is_bootstrapped():
    result = subprocess.run(['juju', 'switch'], stdout=subprocess.PIPE)
    return (
            result.returncode == 0 and
            len(result.stdout.decode().strip()) > 0)

bootstrapped = pytest.mark.skipif(
    not is_bootstrapped(),
    reason='bootstrapped Juju environment required')

def check_statuscode(loops, url):
    for _ in range(loops):
        try:
            r = requests.get(url)
            print(r.status_code)
            assert r.status_code == 200
            return r
        except Exception:
            sleep(5)
            print("ping failed, retrying")
    else:
        raise

def check_jsonresponse(varvalue, varname, json, loops):
    found = False
    for _ in range(loops):
        print(_)
        try:
            print("querying: "+json)
            r = requests.get(json)
            print(r.status_code)
            for v in r.json():
                #print("looking for: "+varvalue)
                print("in: "+varname)
                for key, value in v.items() :
                  if key == varname:
                     print (key, value)
                #print("lookup:"+v)
                if varvalue in v[varname]:
                    found = True
                    print("found entry")
                    break
            else:
                sleep(5)

        except Exception:
             sleep(5)
             print("json lookup failed, retrying")
    return found



class CleanController():
    def __init__(self):
        self._controller = None

    async def __aenter__(self):
        self._controller = Controller()
        await self._controller.connect()
        return self._controller

    async def __aexit__(self, exc_type, exc, tb):
        await self._controller.disconnect()


class CleanModel():
    def __init__(self, bakery_client=None):
        self._controller = None
        self._model = None
        self._model_uuid = None
        self._bakery_client = bakery_client

    async def __aenter__(self):
        model_nonce = uuid.uuid4().hex[-4:]
        self._controller = Controller(
        )
        await self._controller.connect()

        frame = inspect.stack()[1]
        test_name = frame.function.replace('_', '-')
        model_name = 'test-{}-{}-{}'.format(
            test_run_nonce,
            test_name,
            model_nonce,
        )
        self._model = await self._controller.add_model(model_name)

        # save the model UUID in case test closes model
        self._model_uuid = self._model.info.uuid

        return self._model

    async def __aexit__(self, exc_type, exc, tb):
        await self._model.disconnect()
        await self._controller.destroy_model(self._model_uuid)
        await self._controller.disconnect()




# @pytest.fixture()
# async def setup():
#     model_nonce = uuid.uuid4().hex[-4:]
#     model_name = 'test-{}-{}-{}'.format(
#         test_run_nonce,
#         "basicbootstrap",
#         model_nonce,
#     )
#     controller = Controller()
#     await controller.connect()
#     print('adding model')
#     model = await controller.add_model(model_name)
#     return model


