from tempfile import NamedTemporaryFile
import pytest
import requests
from .. import base
import urllib.request as request
from time import sleep

@base.bootstrapped
@pytest.mark.asyncio
async def test_deploy():
    async with base.CleanModel() as model:

        print('deploying zookeeper')
        unit_zookeeper = await model.deploy('zookeeper')

        print('deploying apache-drill')
        unit_drill = await model.deploy('~spiculecharms/apache-drill', channel="edge")  # noqa

        await unit_zookeeper.add_relation('zookeeper', 'apache-drill')
        print('blocking until services are ready')
        await model.block_until(
            lambda: (
                    len(unit_drill.units) > 0 and
                    unit_drill.units[0].workload_status_message ==
                    "Apache Drill up and running." and
                    unit_zookeeper.units[0].public_address is not None
            )
        )

        ip_zookeeper = unit_zookeeper.units[0].public_address
        ip_drill = unit_drill.units[0].public_address
        print('downloading config file')
        with NamedTemporaryFile() as f:
            await unit_drill.units[0].scp_from('/var/snap/apache-drill-spicule/current/drill/conf/drill-override.conf', f.name)  # noqa
            file = f.read().decode("utf-8")
            print(file)
            ip = ip_zookeeper
            check = ip.strip() in file
            assert check


        await model.block_until(
            lambda: (
                    unit_drill.units[0].workload_status_message ==
                    "Apache Drill up and running."
            )
        )


        lc = 10
        print('checking web interface')
        for _ in range(lc):
            try:
                r = requests.get('http://'+ip_drill+':8047')
                print(r.status_code)
                assert r.status_code == 200
                check = 'Bit to Bit Encryption' in r.text
                assert check
                break
            except Exception:
                sleep(5)
                print("ping failed, retrying")
        else:
            raise




