import pytest
from .. import base
from time import sleep
import requests


@base.bootstrapped
@pytest.mark.asyncio
async def test_mysqlrelation():
    async with base.CleanModel() as model:

        print('deploying zookeeper')
        unit_zookeeper = await model.deploy('zookeeper')

        print('deploying mysql')
        unit_mysql = await model.deploy('mysql', series="xenial")

        print('deploying apache-drill')
        unit_drill = await model.deploy('~spiculecharms/apache-drill', channel="edge")  # noqa

        await unit_zookeeper.add_relation('zookeeper', 'apache-drill')
        print('blocking until services are ready')
        await model.block_until(
            lambda: (
                    len(unit_drill.units) > 0 and
                    unit_drill.units[0].workload_status_message ==
                    "Apache Drill up and running." and
                    unit_mysql.units[0].workload_status_message ==
                    "Ready" and
                    unit_zookeeper.units[0].public_address is not None
            )
        )

        sleep(20)
        ip_drill = unit_drill.units[0].public_address
        print('checking storage page for previous mysql sources')
        r = base.check_statuscode(10, 'http://' + ip_drill + ':8047/storage.json')

        found = base.check_jsonresponse('mysql', 'name', 'http://' + ip_drill + ':8047/storage.json', 10)
        
        if not found:
            assert True
        else:
            assert False



        await unit_mysql.add_relation('db', 'apache-drill')
        print('checking storage page for previous mysql sources')
        r = base.check_statuscode(10, 'http://' + ip_drill + ':8047/storage.json')

        found = base.check_jsonresponse('mysql', 'name', 'http://' + ip_drill + ':8047/storage.json', 240)
        if not found:
            assert False
        else:
            assert True

        #ip_mysql = unit_mysql.units[0].public_address
        #for var in r.json():
        #    print(var['name'])
        #    if 'mysql' in var['name'] and ip_mysql in var['url']:
        #        assert True
        #        break
        #else:
        #    assert False

